import { PropsMap, PropsList, CompositeValue } from '@alilc/lowcode-types';
import { Prop, IPropParent } from './prop';
import { Node } from '../node';
import { TransformStage } from '../transform-stage';
export declare const EXTRA_KEY_PREFIX = "___";
export declare function getConvertedExtraKey(key: string): string;
export declare function getOriginalExtraKey(key: string): string;
export declare class Props implements IPropParent {
    readonly id: string;
    private items;
    private get maps();
    readonly path: any[];
    get props(): Props;
    readonly owner: Node;
    /**
     * 元素个数
     */
    get size(): number;
    type: 'map' | 'list';
    constructor(owner: Node, value?: PropsMap | PropsList | null, extras?: object);
    import(value?: PropsMap | PropsList | null, extras?: object): void;
    merge(value: PropsMap, extras?: PropsMap): void;
    export(stage?: TransformStage): {
        props?: PropsMap | PropsList;
        extras?: object;
    };
    /**
     * @deprecated
     */
    private transformToStatic;
    /**
     * 根据 path 路径查询属性
     *
     * @param createIfNone 当没有的时候，是否创建一个
     */
    query(path: string, createIfNone?: boolean): Prop | null;
    /**
     * 获取某个属性, 如果不存在，临时获取一个待写入
     * @param createIfNone 当没有的时候，是否创建一个
     */
    get(path: string, createIfNone?: boolean): Prop | null;
    /**
     * 删除项
     */
    delete(prop: Prop): void;
    /**
     * 删除 key
     */
    deleteKey(key: string): void;
    /**
     * 添加值
     */
    add(value: CompositeValue | null, key?: string | number, spread?: boolean, options?: any): Prop;
    /**
     * 是否存在 key
     */
    has(key: string): boolean;
    /**
     * 迭代器
     */
    [Symbol.iterator](): {
        next(): {
            value: Prop;
        };
    };
    /**
     * 遍历
     */
    forEach(fn: (item: Prop, key: number | string | undefined) => void): void;
    /**
     * 遍历
     */
    map<T>(fn: (item: Prop, key: number | string | undefined) => T): T[] | null;
    filter(fn: (item: Prop, key: number | string | undefined) => boolean): Prop[];
    private purged;
    /**
     * 回收销毁
     */
    purge(): void;
    /**
     * 获取某个属性, 如果不存在，临时获取一个待写入
     * @param createIfNone 当没有的时候，是否创建一个
     */
    getProp(path: string, createIfNone?: boolean): Prop | null;
    /**
     * 获取单个属性值
     */
    getPropValue(path: string): any;
    /**
     * 设置单个属性值
     */
    setPropValue(path: string, value: any): void;
    /**
     * 获取 props 对应的 node
     */
    getNode(): Node<import("@alilc/lowcode-types").NodeSchema>;
    /**
     * @deprecated
     * 获取 props 对应的 node
     */
    toData(): import("@alilc/lowcode-types").CompositeObject | PropsList;
}
