import { engineConfig } from '@alilc/lowcode-editor-core';
import { Hotkey, Project, Skeleton, Setters, Material, Event, editorSymbol, designerSymbol, skeletonSymbol } from '@alilc/lowcode-shell';
import { getLogger } from '@alilc/lowcode-utils';

var PluginContext = function PluginContext(plugins, options) {
  this[editorSymbol] = void 0;
  this[designerSymbol] = void 0;
  this[skeletonSymbol] = void 0;
  this.hotkey = void 0;
  this.project = void 0;
  this.skeleton = void 0;
  this.logger = void 0;
  this.setters = void 0;
  this.material = void 0;
  this.config = void 0;
  this.event = void 0;
  this.plugins = void 0;
  var editor = this[editorSymbol] = plugins.editor;
  var designer = this[designerSymbol] = editor.get('designer');
  var skeleton = this[skeletonSymbol] = editor.get('skeleton');
  var _options$pluginName = options.pluginName,
      pluginName = _options$pluginName === void 0 ? 'anonymous' : _options$pluginName;
  var project = designer === null || designer === void 0 ? void 0 : designer.project;
  this.hotkey = new Hotkey();
  this.project = new Project(project);
  this.skeleton = new Skeleton(skeleton);
  this.setters = new Setters();
  this.material = new Material(editor);
  this.config = engineConfig;
  this.plugins = plugins;
  this.event = new Event(editor, {
    prefix: 'common'
  });
  this.logger = getLogger({
    level: 'warn',
    bizName: "designer:plugin:" + pluginName
  });
};

export { PluginContext as default };