import { CompositeObject, ComponentAction } from '@alilc/lowcode-types';
import Logger from 'zen-logger';
import { Hotkey, Skeleton, Project, Event, Material } from '@alilc/lowcode-shell';
import { EngineConfig } from '@alilc/lowcode-editor-core';
import { MetadataTransducer } from '@alilc/lowcode-designer';
import { Setters } from '../types';
export interface ILowCodePluginConfig {
    name: string;
    dep?: string[];
    init?(): void;
    destroy?(): void;
    exports?(): any;
}
export interface ILowCodePluginCore {
    name: string;
    dep: string[];
    disabled: boolean;
    config: ILowCodePluginConfig;
    logger: Logger;
    on(event: string | symbol, listener: (...args: any[]) => void): any;
    emit(event: string | symbol, ...args: any[]): boolean;
    removeAllListeners(event?: string | symbol): this;
    init(forceInit?: boolean): void;
    isInited(): boolean;
    destroy(): void;
    toProxy(): any;
    setDisabled(flag: boolean): void;
}
interface ILowCodePluginExportsAccessor {
    [propName: string]: any;
}
export declare type ILowCodePlugin = ILowCodePluginCore & ILowCodePluginExportsAccessor;
export interface IDesignerCabin {
    registerMetadataTransducer: (transducer: MetadataTransducer, level: number, id?: string) => void;
    addBuiltinComponentAction: (action: ComponentAction) => void;
    removeBuiltinComponentAction: (actionName: string) => void;
}
export interface ILowCodePluginContext {
    skeleton: Skeleton;
    hotkey: Hotkey;
    logger: Logger;
    plugins: ILowCodePluginManager;
    setters: Setters;
    config: EngineConfig;
    material: Material;
    event: Event;
    project: Project;
}
interface ILowCodePluginManagerPluginAccessor {
    [pluginName: string]: ILowCodePlugin | any;
}
export interface ILowCodePluginManagerCore {
    register(pluginConfigCreator: (ctx: ILowCodePluginContext, pluginOptions?: any) => ILowCodePluginConfig, pluginOptions?: any, options?: CompositeObject): Promise<void>;
    get(pluginName: string): ILowCodePlugin | undefined;
    getAll(): ILowCodePlugin[];
    has(pluginName: string): boolean;
    delete(pluginName: string): any;
    setDisabled(pluginName: string, flag: boolean): void;
    dispose(): void;
}
export declare type ILowCodePluginManager = ILowCodePluginManagerCore & ILowCodePluginManagerPluginAccessor;
export declare type LowCodeRegisterOptions = {
    autoInit?: boolean;
    override?: boolean;
};
export declare type PluginContextOptions = {
    pluginName: string;
};
export {};
