import { ReactElement } from 'react';
import { ComponentMetadata, NpmInfo, NodeData, NodeSchema, ComponentAction, TransformedComponentMetadata, I18nData, LiveTextEditingConfig, FieldConfig } from '@alilc/lowcode-types';
import { Node, ParentalNode } from './document';
import { Designer } from './designer';
export declare class ComponentMeta {
    readonly designer: Designer;
    readonly isComponentMeta = true;
    private _npm?;
    private emitter;
    get npm(): any;
    set npm(_npm: any);
    private _componentName?;
    get componentName(): string;
    private _isContainer?;
    get isContainer(): boolean;
    get isMinimalRenderUnit(): boolean;
    private _isModal?;
    get isModal(): boolean;
    private _descriptor?;
    get descriptor(): string | undefined;
    private _rootSelector?;
    get rootSelector(): string | undefined;
    private _transformedMetadata?;
    get configure(): FieldConfig[];
    private _liveTextEditing?;
    get liveTextEditing(): LiveTextEditingConfig[];
    private _isTopFixed?;
    get isTopFixed(): boolean;
    private parentWhitelist?;
    private childWhitelist?;
    private _title?;
    private _isMinimalRenderUnit?;
    get title(): string | I18nData | ReactElement;
    get icon(): any;
    private _acceptable?;
    get acceptable(): boolean;
    constructor(designer: Designer, metadata: ComponentMetadata);
    setNpm(info: NpmInfo): void;
    private parseMetadata;
    refreshMetadata(): void;
    private transformMetadata;
    isRootComponent(includeBlock?: boolean): boolean;
    get availableActions(): ComponentAction[];
    setMetadata(metadata: ComponentMetadata): void;
    getMetadata(): TransformedComponentMetadata;
    checkNestingUp(my: Node | NodeData, parent: ParentalNode): boolean;
    checkNestingDown(my: Node, target: Node | NodeSchema | NodeSchema[]): any;
    onMetadataChange(fn: (args: any) => void): () => void;
    prototype?: any;
}
export declare function isComponentMeta(obj: any): obj is ComponentMeta;
export interface MetadataTransducer {
    (prev: TransformedComponentMetadata): TransformedComponentMetadata;
    /**
     * 0 - 9   system
     * 10 - 99 builtin-plugin
     * 100 -   app & plugin
     */
    level?: number;
    /**
     * use to replace TODO
     */
    id?: string;
}
export declare function registerMetadataTransducer(transducer: MetadataTransducer, level?: number, id?: string): void;
export declare function getRegisteredMetadataTransducers(): MetadataTransducer[];
export declare function removeBuiltinComponentAction(name: string): void;
export declare function addBuiltinComponentAction(action: ComponentAction): void;
export declare function modifyBuiltinComponentAction(actionName: string, handle: (action: ComponentAction) => void): void;
