"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.LowCodePluginManager = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _lowcodeUtils = require("@alilc/lowcode-utils");

var _plugin = require("./plugin");

var _pluginContext = _interopRequireDefault(require("./plugin-context"));

var _utils = require("../utils");

var _sequencify2 = _interopRequireDefault(require("./sequencify"));

function _createForOfIteratorHelperLoose(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (it) return (it = it.call(o)).next.bind(it); if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; return function () { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var logger = (0, _lowcodeUtils.getLogger)({
  level: 'warn',
  bizName: 'designer:pluginManager'
});

var LowCodePluginManager = /*#__PURE__*/function () {
  function LowCodePluginManager(editor) {
    this.plugins = [];
    this.pluginsMap = new Map();
    this.editor = void 0;
    this.editor = editor;
  }

  var _proto = LowCodePluginManager.prototype;

  _proto._getLowCodePluginContext = function _getLowCodePluginContext(options) {
    return new _pluginContext["default"](this, options);
  };

  _proto.register = /*#__PURE__*/function () {
    var _register = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(pluginConfigCreator, pluginOptions, options) {
      var ctx, config, allowOverride, originalPlugin, plugin;
      return _regenerator["default"].wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              ctx = this._getLowCodePluginContext({
                pluginName: pluginConfigCreator.pluginName
              });
              config = pluginConfigCreator(ctx, pluginOptions);
              (0, _utils.invariant)(pluginConfigCreator.pluginName || config.name, 'pluginConfigCreator.pluginName required', config);
              allowOverride = (options === null || options === void 0 ? void 0 : options.override) === true;

              if (!this.pluginsMap.has(config.name)) {
                _context.next = 13;
                break;
              }

              if (allowOverride) {
                _context.next = 9;
                break;
              }

              throw new Error("Plugin with name " + config.name + " exists");

            case 9:
              // clear existing plugin
              originalPlugin = this.pluginsMap.get(config.name);
              logger.log('plugin override, originalPlugin with name ', config.name, ' will be destroyed, config:', originalPlugin === null || originalPlugin === void 0 ? void 0 : originalPlugin.config);
              originalPlugin === null || originalPlugin === void 0 ? void 0 : originalPlugin.destroy();
              this.pluginsMap["delete"](config.name);

            case 13:
              plugin = new _plugin.LowCodePlugin(this, config, pluginOptions);

              if (!(options !== null && options !== void 0 && options.autoInit)) {
                _context.next = 17;
                break;
              }

              _context.next = 17;
              return plugin.init();

            case 17:
              this.plugins.push(plugin);
              this.pluginsMap.set(plugin.name, plugin);
              logger.log('plugin registered with config:', config, ', options:', pluginOptions);

            case 20:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function register(_x, _x2, _x3) {
      return _register.apply(this, arguments);
    }

    return register;
  }();

  _proto.get = function get(pluginName) {
    return this.pluginsMap.get(pluginName);
  };

  _proto.getAll = function getAll() {
    return this.plugins;
  };

  _proto.has = function has(pluginName) {
    return this.pluginsMap.has(pluginName);
  };

  _proto["delete"] = /*#__PURE__*/function () {
    var _delete2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(pluginName) {
      var idx, plugin;
      return _regenerator["default"].wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              idx = this.plugins.findIndex(function (plugin) {
                return plugin.name === pluginName;
              });

              if (!(idx === -1)) {
                _context2.next = 3;
                break;
              }

              return _context2.abrupt("return", false);

            case 3:
              plugin = this.plugins[idx];
              _context2.next = 6;
              return plugin.destroy();

            case 6:
              this.plugins.splice(idx, 1);
              return _context2.abrupt("return", this.pluginsMap["delete"](pluginName));

            case 8:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    function _delete(_x4) {
      return _delete2.apply(this, arguments);
    }

    return _delete;
  }();

  _proto.init = /*#__PURE__*/function () {
    var _init = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3() {
      var pluginNames, pluginObj, _sequencify, missingTasks, sequence, _iterator, _step, pluginName;

      return _regenerator["default"].wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              pluginNames = [];
              pluginObj = {};
              this.plugins.forEach(function (plugin) {
                pluginNames.push(plugin.name);
                pluginObj[plugin.name] = plugin;
              });
              _sequencify = (0, _sequencify2["default"])(pluginObj, pluginNames), missingTasks = _sequencify.missingTasks, sequence = _sequencify.sequence;
              (0, _utils.invariant)(!missingTasks.length, 'plugin dependency missing', missingTasks);
              logger.log('load plugin sequence:', sequence);
              _iterator = _createForOfIteratorHelperLoose(sequence);

            case 7:
              if ((_step = _iterator()).done) {
                _context3.next = 20;
                break;
              }

              pluginName = _step.value;
              _context3.prev = 9;
              _context3.next = 12;
              return this.pluginsMap.get(pluginName).init();

            case 12:
              _context3.next = 18;
              break;

            case 14:
              _context3.prev = 14;
              _context3.t0 = _context3["catch"](9);
              logger.error("Failed to init plugin:" + pluginName + ", it maybe affect those plugins which depend on this.");
              logger.error(_context3.t0);

            case 18:
              _context3.next = 7;
              break;

            case 20:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3, this, [[9, 14]]);
    }));

    function init() {
      return _init.apply(this, arguments);
    }

    return init;
  }();

  _proto.destroy = /*#__PURE__*/function () {
    var _destroy = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4() {
      var _iterator2, _step2, plugin;

      return _regenerator["default"].wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _iterator2 = _createForOfIteratorHelperLoose(this.plugins);

            case 1:
              if ((_step2 = _iterator2()).done) {
                _context4.next = 7;
                break;
              }

              plugin = _step2.value;
              _context4.next = 5;
              return plugin.destroy();

            case 5:
              _context4.next = 1;
              break;

            case 7:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4, this);
    }));

    function destroy() {
      return _destroy.apply(this, arguments);
    }

    return destroy;
  }();

  _proto.toProxy = function toProxy() {
    return new Proxy(this, {
      get: function get(target, prop, receiver) {
        if (target.pluginsMap.has(prop)) {
          var _target$pluginsMap$ge;

          // 禁用态的插件，直接返回 undefined
          if (target.pluginsMap.get(prop).disabled) {
            return undefined;
          }

          return (_target$pluginsMap$ge = target.pluginsMap.get(prop)) === null || _target$pluginsMap$ge === void 0 ? void 0 : _target$pluginsMap$ge.toProxy();
        }

        return Reflect.get(target, prop, receiver);
      }
    });
  };

  _proto.setDisabled = function setDisabled(pluginName, flag) {
    var _this$pluginsMap$get;

    if (flag === void 0) {
      flag = true;
    }

    logger.warn("plugin:" + pluginName + " has been set disable:" + flag);
    (_this$pluginsMap$get = this.pluginsMap.get(pluginName)) === null || _this$pluginsMap$get === void 0 ? void 0 : _this$pluginsMap$get.setDisabled(flag);
  };

  _proto.dispose = /*#__PURE__*/function () {
    var _dispose = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee5() {
      return _regenerator["default"].wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return this.destroy();

            case 2:
              this.plugins = [];
              this.pluginsMap.clear();

            case 4:
            case "end":
              return _context5.stop();
          }
        }
      }, _callee5, this);
    }));

    function dispose() {
      return _dispose.apply(this, arguments);
    }

    return dispose;
  }();

  (0, _createClass2["default"])(LowCodePluginManager, [{
    key: "size",
    get: function get() {
      return this.pluginsMap.size;
    }
  }]);
  return LowCodePluginManager;
}();

exports.LowCodePluginManager = LowCodePluginManager;